@@ -23,7 +23,7 @@ from sphinx.errors import ThemeError
 from sphinx.locale import __
 from sphinx.util import split_index_msg, logging, status_iterator
 from sphinx.util.console import bold  # type: ignore
-from sphinx.util.i18n import find_catalog
+from sphinx.util.i18n import docname_to_domain
 from sphinx.util.nodes import extract_messages, traverse_translatable_index
 from sphinx.util.osutil import relpath, ensuredir, canon_path
 from sphinx.util.tags import Tags
@@ -141,7 +141,7 @@ class I18nBuilder(Builder):
 
     def write_doc(self, docname, doctree):
         # type: (str, nodes.document) -> None
-        catalog = self.catalogs[find_catalog(docname, self.config.gettext_compact)]
+        catalog = self.catalogs[docname_to_domain(docname, self.config.gettext_compact)]
 
         for toctree in self.env.tocs[docname].traverse(addnodes.toctree):
             for node, msg in extract_messages(toctree):
