@@ -9,6 +9,7 @@
 """
 
 import os
+import warnings
 from os import path
 
 from docutils.frontend import OptionParser
@@ -16,18 +17,12 @@ from docutils.frontend import OptionParser
 import sphinx.builders.latex.nodes  # NOQA  # Workaround: import this before writer to avoid ImportError
 from sphinx import package_dir, addnodes, highlighting
 from sphinx.builders import Builder
-from sphinx.builders.latex.transforms import (
-    BibliographyTransform, CitationReferenceTransform, MathReferenceTransform,
-    FootnoteDocnameUpdater, LaTeXFootnoteTransform, LiteralBlockTransform,
-    ShowUrlsTransform, DocumentTargetTransform, IndexInSectionTitleTransform,
-)
 from sphinx.builders.latex.util import ExtBabel
 from sphinx.config import ENUM
-from sphinx.environment import NoUri
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.environment.adapters.asset import ImageAdapter
-from sphinx.errors import SphinxError
+from sphinx.errors import NoUri, SphinxError
 from sphinx.locale import _, __
-from sphinx.transforms import SphinxTransformer
 from sphinx.util import texescape, logging, progress_message, status_iterator
 from sphinx.util.console import bold, darkgreen  # type: ignore
 from sphinx.util.docutils import SphinxFileOutput, new_document
@@ -185,12 +180,10 @@ class LaTeXBuilder(Builder):
         # Add special settings for latex_engine
         self.context.update(ADDITIONAL_SETTINGS.get(self.config.latex_engine, {}))
 
-        # for xelatex+French, don't use polyglossia by default
-        if self.config.latex_engine == 'xelatex':
-            if self.config.language:
-                if self.config.language[:2] == 'fr':
-                    self.context['polyglossia'] = ''
-                    self.context['babel'] = r'\usepackage{babel}'
+        # Add special settings for (latex_engine, language_code)
+        if self.config.language:
+            key = (self.config.latex_engine, self.config.language[:2])
+            self.context.update(ADDITIONAL_SETTINGS.get(key, {}))
 
         # Apply extension settings to context
         self.context['packages'] = self.usepackages
@@ -265,7 +258,6 @@ class LaTeXBuilder(Builder):
                     docname, toctree_only,
                     appendices=((docclass != 'howto') and self.config.latex_appendices or []))
                 doctree['tocdepth'] = tocdepth
-                self.apply_transforms(doctree)
                 self.post_process_images(doctree)
                 self.update_doc_context(title, author)
 
@@ -341,15 +333,8 @@ class LaTeXBuilder(Builder):
 
     def apply_transforms(self, doctree):
         # type: (nodes.document) -> None
-        transformer = SphinxTransformer(doctree)
-        transformer.set_environment(self.env)
-        transformer.add_transforms([BibliographyTransform,
-                                    ShowUrlsTransform,
-                                    LaTeXFootnoteTransform,
-                                    LiteralBlockTransform,
-                                    DocumentTargetTransform,
-                                    IndexInSectionTitleTransform])
-        transformer.apply_transforms()
+        warnings.warn('LaTeXBuilder.apply_transforms() is deprecated.',
+                      RemovedInSphinx40Warning)
 
     def finish(self):
         # type: () -> None
@@ -454,6 +439,8 @@ def default_latex_engine(config):
     """ Better default latex_engine settings for specific languages. """
     if config.language == 'ja':
         return 'platex'
+    elif (config.language or '').startswith('zh'):
+        return 'xelatex'
     else:
         return 'pdflatex'
 
@@ -486,11 +473,10 @@ def default_latex_documents(config):
 
 def setup(app):
     # type: (Sphinx) -> Dict[str, Any]
+    app.setup_extension('sphinx.builders.latex.transforms')
+
     app.add_builder(LaTeXBuilder)
-    app.add_post_transform(CitationReferenceTransform)
-    app.add_post_transform(MathReferenceTransform)
     app.connect('config-inited', validate_config_values)
-    app.add_transform(FootnoteDocnameUpdater)
 
     app.add_config_value('latex_engine', default_latex_engine, None,
                          ENUM('pdflatex', 'xelatex', 'lualatex', 'platex'))
