@@ -9,12 +9,16 @@
 """
 
 import re
+from typing import cast
+
+from docutils.parsers.rst import directives
 
 from sphinx import addnodes
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType
-from sphinx.locale import _
+from sphinx.locale import _, __
 from sphinx.roles import XRefRole
+from sphinx.util import logging
 from sphinx.util.nodes import make_refnode
 
 if False:
@@ -26,6 +30,8 @@ if False:
     from sphinx.environment import BuildEnvironment  # NOQA
 
 
+logger = logging.getLogger(__name__)
+
 dir_sig_re = re.compile(r'\.\. (.+?)::(.*)$')
 
 
@@ -43,14 +49,9 @@ class ReSTMarkup(ObjectDescription):
             signode['first'] = (not self.names)
             self.state.document.note_explicit_target(signode)
 
-            objects = self.env.domaindata['rst']['objects']
-            key = (self.objtype, name)
-            if key in objects:
-                self.state_machine.reporter.warning(
-                    'duplicate description of %s %s, ' % (self.objtype, name) +
-                    'other instance in ' + self.env.doc2path(objects[key]),
-                    line=self.lineno)
-            objects[key] = self.env.docname
+            domain = cast(ReSTDomain, self.env.get_domain('rst'))
+            domain.note_object(self.objtype, name, location=(self.env.docname, self.lineno))
+
         indextext = self.get_index_text(self.objtype, name)
         if indextext:
             self.indexnode['entries'].append(('single', indextext,
@@ -58,10 +59,6 @@ class ReSTMarkup(ObjectDescription):
 
     def get_index_text(self, objectname, name):
         # type: (str, str) -> str
-        if self.objtype == 'directive':
-            return _('%s (directive)') % name
-        elif self.objtype == 'role':
-            return _('%s (role)') % name
         return ''
 
 
@@ -80,7 +77,10 @@ def parse_directive(d):
     if not m:
         return (dir, '')
     parsed_dir, parsed_args = m.groups()
-    return (parsed_dir.strip(), ' ' + parsed_args.strip())
+    if parsed_args.strip():
+        return (parsed_dir.strip(), ' ' + parsed_args.strip())
+    else:
+        return (parsed_dir.strip(), '')
 
 
 class ReSTDirective(ReSTMarkup):
@@ -96,6 +96,80 @@ class ReSTDirective(ReSTMarkup):
             signode += addnodes.desc_addname(args, args)
         return name
 
+    def get_index_text(self, objectname, name):
+        # type: (str, str) -> str
+        return _('%s (directive)') % name
+
+    def before_content(self):
+        # type: () -> None
+        if self.names:
+            directives = self.env.ref_context.setdefault('rst:directives', [])
+            directives.append(self.names[0])
+
+    def after_content(self):
+        # type: () -> None
+        directives = self.env.ref_context.setdefault('rst:directives', [])
+        if directives:
+            directives.pop()
+
+
+class ReSTDirectiveOption(ReSTMarkup):
+    """
+    Description of an option for reST directive.
+    """
+    option_spec = ReSTMarkup.option_spec.copy()
+    option_spec.update({
+        'type': directives.unchanged,
+    })
+
+    def handle_signature(self, sig, signode):
+        # type: (str, addnodes.desc_signature) -> str
+        try:
+            name, argument = re.split(r'\s*:\s+', sig.strip(), 1)
+        except ValueError:
+            name, argument = sig, None
+
+        signode += addnodes.desc_name(':%s:' % name, ':%s:' % name)
+        if argument:
+            signode += addnodes.desc_annotation(' ' + argument, ' ' + argument)
+        if self.options.get('type'):
+            text = ' (%s)' % self.options['type']
+            signode += addnodes.desc_annotation(text, text)
+        return name
+
+    def add_target_and_index(self, name, sig, signode):
+        # type: (str, str, addnodes.desc_signature) -> None
+        directive_name = self.current_directive
+        targetname = '-'.join([self.objtype, self.current_directive, name])
+        if targetname not in self.state.document.ids:
+            signode['names'].append(targetname)
+            signode['ids'].append(targetname)
+            signode['first'] = (not self.names)
+            self.state.document.note_explicit_target(signode)
+
+            objname = ':'.join(filter(None, [directive_name, name]))
+            domain = cast(ReSTDomain, self.env.get_domain('rst'))
+            domain.note_object(self.objtype, objname, location=(self.env.docname, self.lineno))
+
+        if directive_name:
+            key = name[0].upper()
+            pair = [_('%s (directive)') % directive_name,
+                    _(':%s: (directive option)') % name]
+            self.indexnode['entries'].append(('pair', '; '.join(pair), targetname, '', key))
+        else:
+            key = name[0].upper()
+            text = _(':%s: (directive option)') % name
+            self.indexnode['entries'].append(('single', text, targetname, '', key))
+
+    @property
+    def current_directive(self):
+        # type: () -> str
+        directives = self.env.ref_context.get('rst:directives')
+        if directives:
+            return directives[-1]
+        else:
+            return ''
+
 
 class ReSTRole(ReSTMarkup):
     """
@@ -106,6 +180,10 @@ class ReSTRole(ReSTMarkup):
         signode += addnodes.desc_name(':%s:' % sig, ':%s:' % sig)
         return sig
 
+    def get_index_text(self, objectname, name):
+        # type: (str, str) -> str
+        return _('%s (role)') % name
+
 
 class ReSTDomain(Domain):
     """ReStructuredText domain."""
@@ -113,11 +191,13 @@ class ReSTDomain(Domain):
     label = 'reStructuredText'
 
     object_types = {
-        'directive': ObjType(_('directive'), 'dir'),
-        'role':      ObjType(_('role'),      'role'),
+        'directive':        ObjType(_('directive'), 'dir'),
+        'directive:option': ObjType(_('directive-option'), 'dir'),
+        'role':             ObjType(_('role'),      'role'),
     }
     directives = {
         'directive': ReSTDirective,
+        'directive:option': ReSTDirectiveOption,
         'role':      ReSTRole,
     }
     roles = {
@@ -126,42 +206,54 @@ class ReSTDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-    }  # type: Dict[str, Dict[str, Tuple[str, ObjType]]]
+    }  # type: Dict[str, Dict[Tuple[str, str], str]]
+
+    @property
+    def objects(self):
+        # type: () -> Dict[Tuple[str, str], str]
+        return self.data.setdefault('objects', {})  # (objtype, fullname) -> docname
+
+    def note_object(self, objtype, name, location=None):
+        # type: (str, str, Any) -> None
+        if (objtype, name) in self.objects:
+            docname = self.objects[objtype, name]
+            logger.warning(__('duplicate description of %s %s, other instance in %s') %
+                           (objtype, name, docname), location=location)
+
+        self.objects[objtype, name] = self.env.docname
 
     def clear_doc(self, docname):
         # type: (str) -> None
-        for (typ, name), doc in list(self.data['objects'].items()):
+        for (typ, name), doc in list(self.objects.items()):
             if doc == docname:
-                del self.data['objects'][typ, name]
+                del self.objects[typ, name]
 
     def merge_domaindata(self, docnames, otherdata):
         # type: (List[str], Dict) -> None
         # XXX check duplicates
         for (typ, name), doc in otherdata['objects'].items():
             if doc in docnames:
-                self.data['objects'][typ, name] = doc
+                self.objects[typ, name] = doc
 
     def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
         # type: (BuildEnvironment, str, Builder, str, str, addnodes.pending_xref, nodes.Element) -> nodes.Element  # NOQA
-        objects = self.data['objects']
         objtypes = self.objtypes_for_role(typ)
         for objtype in objtypes:
-            if (objtype, target) in objects:
-                return make_refnode(builder, fromdocname,
-                                    objects[objtype, target],
+            todocname = self.objects.get((objtype, target))
+            if todocname:
+                return make_refnode(builder, fromdocname, todocname,
                                     objtype + '-' + target,
                                     contnode, target + ' ' + objtype)
         return None
 
     def resolve_any_xref(self, env, fromdocname, builder, target, node, contnode):
         # type: (BuildEnvironment, str, Builder, str, addnodes.pending_xref, nodes.Element) -> List[Tuple[str, nodes.Element]]  # NOQA
-        objects = self.data['objects']
         results = []  # type: List[Tuple[str, nodes.Element]]
         for objtype in self.object_types:
-            if (objtype, target) in self.data['objects']:
+            todocname = self.objects.get((objtype, target))
+            if todocname:
                 results.append(('rst:' + self.role_for_objtype(objtype),
-                                make_refnode(builder, fromdocname,
-                                             objects[objtype, target],
+                                make_refnode(builder, fromdocname, todocname,
                                              objtype + '-' + target,
                                              contnode, target + ' ' + objtype)))
         return results
