@@ -1,16 +1,16 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.environment.adapters.toctree
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     Toctree adapter for sphinx.environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Iterable, cast
+
 from docutils import nodes
-from six import iteritems
 
 from sphinx import addnodes
 from sphinx.locale import __
@@ -27,13 +27,13 @@ if False:
 logger = logging.getLogger(__name__)
 
 
-class TocTree(object):
+class TocTree:
     def __init__(self, env):
         # type: (BuildEnvironment) -> None
         self.env = env
 
     def note(self, docname, toctreenode):
-        # type: (unicode, addnodes.toctree) -> None
+        # type: (str, addnodes.toctree) -> None
         """Note a TOC tree directive in a document and gather information about
         file relations from it.
         """
@@ -50,7 +50,7 @@ class TocTree(object):
 
     def resolve(self, docname, builder, toctree, prune=True, maxdepth=0,
                 titles_only=False, collapse=False, includehidden=False):
-        # type: (unicode, Builder, addnodes.toctree, bool, int, bool, bool, bool) -> nodes.Node
+        # type: (str, Builder, addnodes.toctree, bool, int, bool, bool, bool) -> nodes.Element
         """Resolve a *toctree* node into individual bullet lists with titles
         as items, returning None (if no containing titles are found) or
         a new node.
@@ -87,7 +87,7 @@ class TocTree(object):
         excluded = Matcher(self.env.config.exclude_patterns)
 
         def _toctree_add_classes(node, depth):
-            # type: (nodes.Node, int) -> None
+            # type: (nodes.Element, int) -> None
             """Add 'toctree-l%d' and 'current' classes to the toctree."""
             for subnode in node.children:
                 if isinstance(subnode, (addnodes.compact_paragraph,
@@ -105,7 +105,7 @@ class TocTree(object):
                         if not subnode['anchorname']:
                             # give the whole branch a 'current' class
                             # (useful for styling it differently)
-                            branchnode = subnode
+                            branchnode = subnode  # type: nodes.Element
                             while branchnode:
                                 branchnode['classes'].append('current')
                                 branchnode = branchnode.parent
@@ -118,10 +118,10 @@ class TocTree(object):
                             subnode = subnode.parent
 
         def _entries_from_toctree(toctreenode, parents, separate=False, subtree=False):
-            # type: (addnodes.toctree, List[nodes.Node], bool, bool) -> List[nodes.Node]
+            # type: (addnodes.toctree, List[str], bool, bool) -> List[nodes.Element]
             """Return TOC entries for a toctree node."""
             refs = [(e[0], e[1]) for e in toctreenode['entries']]
-            entries = []
+            entries = []  # type: List[nodes.Element]
             for (title, ref) in refs:
                 try:
                     refdoc = None
@@ -184,14 +184,20 @@ class TocTree(object):
                     # if titles_only is given, only keep the main title and
                     # sub-toctrees
                     if titles_only:
+                        # children of toc are:
+                        # - list_item + compact_paragraph + (reference and subtoc)
+                        # - only + subtoc
+                        # - toctree
+                        children = cast(Iterable[nodes.Element], toc)
+
                         # delete everything but the toplevel title(s)
                         # and toctrees
-                        for toplevel in toc:
+                        for toplevel in children:
                             # nodes with length 1 don't have any children anyway
                             if len(toplevel) > 1:
                                 subtrees = toplevel.traverse(addnodes.toctree)
                                 if subtrees:
-                                    toplevel[1][:] = subtrees
+                                    toplevel[1][:] = subtrees  # type: ignore
                                 else:
                                     toplevel.pop(1)
                     # resolve all sub-toctrees
@@ -199,16 +205,17 @@ class TocTree(object):
                         if not (subtocnode.get('hidden', False) and
                                 not includehidden):
                             i = subtocnode.parent.index(subtocnode) + 1
-                            for item in _entries_from_toctree(
+                            for entry in _entries_from_toctree(
                                     subtocnode, [refdoc] + parents,
                                     subtree=True):
-                                subtocnode.parent.insert(i, item)
+                                subtocnode.parent.insert(i, entry)
                                 i += 1
                             subtocnode.parent.remove(subtocnode)
                     if separate:
                         entries.append(toc)
                     else:
-                        entries.extend(toc.children)
+                        children = cast(Iterable[nodes.Element], toc)
+                        entries.extend(children)
             if not subtree and not separate:
                 ret = nodes.bullet_list()
                 ret += entries
@@ -237,8 +244,8 @@ class TocTree(object):
             caption_node.rawsource = toctree['rawcaption']
             if hasattr(toctree, 'uid'):
                 # move uid to caption_node to translate it
-                caption_node.uid = toctree.uid
-                del toctree.uid
+                caption_node.uid = toctree.uid  # type: ignore
+                del toctree.uid  # type: ignore
             newnode += caption_node
         newnode.extend(tocentries)
         newnode['toctree'] = True
@@ -247,7 +254,7 @@ class TocTree(object):
         _toctree_add_classes(newnode, 1)
         self._toctree_prune(newnode, 1, prune and maxdepth or 0, collapse)
 
-        if len(newnode[-1]) == 0:  # No titles found
+        if isinstance(newnode[-1], nodes.Element) and len(newnode[-1]) == 0:  # No titles found
             return None
 
         # set the target paths in the toctrees (they are not known at TOC
@@ -259,12 +266,12 @@ class TocTree(object):
         return newnode
 
     def get_toctree_ancestors(self, docname):
-        # type: (unicode) -> List[unicode]
+        # type: (str) -> List[str]
         parent = {}
-        for p, children in iteritems(self.env.toctree_includes):
+        for p, children in self.env.toctree_includes.items():
             for child in children:
                 parent[child] = p
-        ancestors = []  # type: List[unicode]
+        ancestors = []  # type: List[str]
         d = docname
         while d in parent and d not in ancestors:
             ancestors.append(d)
@@ -272,7 +279,7 @@ class TocTree(object):
         return ancestors
 
     def _toctree_prune(self, node, depth, maxdepth, collapse=False):
-        # type: (nodes.Node, int, int, bool) -> None
+        # type: (nodes.Element, int, int, bool) -> None
         """Utility: Cut a TOC at a specified depth."""
         for subnode in node.children[:]:
             if isinstance(subnode, (addnodes.compact_paragraph,
@@ -294,7 +301,7 @@ class TocTree(object):
                         self._toctree_prune(subnode, depth + 1, maxdepth,  collapse)
 
     def get_toc_for(self, docname, builder):
-        # type: (unicode, Builder) -> Dict[unicode, nodes.Node]
+        # type: (str, Builder) -> nodes.Node
         """Return a TOC nodetree -- for use on the same page only!"""
         tocdepth = self.env.metadata[docname].get('tocdepth', 0)
         try:
@@ -310,10 +317,10 @@ class TocTree(object):
         return toc
 
     def get_toctree_for(self, docname, builder, collapse, **kwds):
-        # type: (unicode, Builder, bool, Any) -> nodes.Node
+        # type: (str, Builder, bool, Any) -> nodes.Element
         """Return the global TOC nodetree."""
         doctree = self.env.get_doctree(self.env.config.master_doc)
-        toctrees = []
+        toctrees = []  # type: List[nodes.Element]
         if 'includehidden' not in kwds:
             kwds['includehidden'] = True
         if 'maxdepth' not in kwds:
