@@ -1,19 +1,14 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.napoleon
     ~~~~~~~~~~~~~~~~~~~
 
     Support for NumPy and Google style docstrings.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
-
-from six import PY2, iteritems
-
-import sphinx
+from sphinx import __display_version__ as __version__
 from sphinx.application import Sphinx
 from sphinx.ext.napoleon.docstring import GoogleDocstring, NumpyDocstring
 
@@ -22,7 +17,7 @@ if False:
     from typing import Any, Dict, List  # NOQA
 
 
-class Config(object):
+class Config:
     """Sphinx napoleon extension settings in `conf.py`.
 
     Listed below are all the settings used by napoleon and their default
@@ -176,10 +171,10 @@ class Config(object):
 
             .. attribute:: attr1
 
-               *int*
-
                Description of `attr1`
 
+               :type: int
+
     napoleon_use_param : :obj:`bool` (Defaults to True)
         True to use a ``:param:`` role for each function parameter. False to
         use a single ``:parameters:`` role for all the parameters.
@@ -274,14 +269,14 @@ class Config(object):
 
     def __init__(self, **settings):
         # type: (Any) -> None
-        for name, (default, rebuild) in iteritems(self._config_values):
+        for name, (default, rebuild) in self._config_values.items():
             setattr(self, name, default)
-        for name, value in iteritems(settings):
+        for name, value in settings.items():
             setattr(self, name, value)
 
 
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     """Sphinx extension setup function.
 
     When the extension is loaded, Sphinx imports this module and executes
@@ -304,7 +299,8 @@ def setup(app):
 
     """
     if not isinstance(app, Sphinx):
-        return  # probably called by tests
+        # probably called by tests
+        return {'version': __version__, 'parallel_read_safe': True}
 
     _patch_python_domain()
 
@@ -312,9 +308,9 @@ def setup(app):
     app.connect('autodoc-process-docstring', _process_docstring)
     app.connect('autodoc-skip-member', _skip_member)
 
-    for name, (default, rebuild) in iteritems(Config._config_values):
+    for name, (default, rebuild) in Config._config_values.items():
         app.add_config_value(name, default, rebuild)
-    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
+    return {'version': __version__, 'parallel_read_safe': True}
 
 
 def _patch_python_domain():
@@ -338,7 +334,7 @@ def _patch_python_domain():
 
 
 def _process_docstring(app, what, name, obj, options, lines):
-    # type: (Sphinx, unicode, unicode, Any, Any, List[unicode]) -> None
+    # type: (Sphinx, str, str, Any, Any, List[str]) -> None
     """Process the docstring for a given python object.
 
     Called when autodoc has read and processed a docstring. `lines` is a list
@@ -388,7 +384,7 @@ def _process_docstring(app, what, name, obj, options, lines):
 
 
 def _skip_member(app, what, name, obj, skip, options):
-    # type: (Sphinx, unicode, unicode, Any, bool, Any) -> bool
+    # type: (Sphinx, str, str, Any, bool, Any) -> bool
     """Determine if private and special class members are included in docs.
 
     The following settings in conf.py determine if private and special class
@@ -435,34 +431,26 @@ def _skip_member(app, what, name, obj, skip, options):
     if name != '__weakref__' and has_doc and is_member:
         cls_is_owner = False
         if what == 'class' or what == 'exception':
-            if PY2:
-                cls = getattr(obj, 'im_class', getattr(obj, '__objclass__',
-                              None))
-                cls_is_owner = (cls and hasattr(cls, name) and
-                                name in cls.__dict__)
-            elif sys.version_info >= (3, 3):
-                qualname = getattr(obj, '__qualname__', '')
-                cls_path, _, _ = qualname.rpartition('.')
-                if cls_path:
-                    try:
-                        if '.' in cls_path:
-                            import importlib
-                            import functools
+            qualname = getattr(obj, '__qualname__', '')
+            cls_path, _, _ = qualname.rpartition('.')
+            if cls_path:
+                try:
+                    if '.' in cls_path:
+                        import importlib
+                        import functools
 
-                            mod = importlib.import_module(obj.__module__)
-                            mod_path = cls_path.split('.')
-                            cls = functools.reduce(getattr, mod_path, mod)
-                        else:
-                            cls = obj.__globals__[cls_path]
-                    except Exception:
-                        cls_is_owner = False
+                        mod = importlib.import_module(obj.__module__)
+                        mod_path = cls_path.split('.')
+                        cls = functools.reduce(getattr, mod_path, mod)
                     else:
-                        cls_is_owner = (cls and hasattr(cls, name) and
-                                        name in cls.__dict__)
-                else:
+                        cls = obj.__globals__[cls_path]
+                except Exception:
                     cls_is_owner = False
+                else:
+                    cls_is_owner = (cls and hasattr(cls, name) and  # type: ignore
+                                    name in cls.__dict__)
             else:
-                cls_is_owner = True
+                cls_is_owner = False
 
         if what == 'module' or cls_is_owner:
             is_init = (name == '__init__')
