@@ -9,15 +9,18 @@
     :license: BSD, see LICENSE for details.
 """
 
+import warnings
 from typing import List, Tuple, cast
 
 from docutils import nodes
 
 from sphinx import addnodes
+from sphinx.deprecation import RemovedInSphinx40Warning
 
 if False:
     # For type annotation
     from typing import Any, Dict, Type, Union  # NOQA
+    from sphinx.directive import ObjectDescription  # NOQA
     from sphinx.environment import BuildEnvironment  # NOQA
     from sphinx.util.typing import TextlikeNode  # NOQA
 
@@ -244,15 +247,14 @@ class DocFieldTransformer:
     typemap = None  # type: Dict[str, Tuple[Field, bool]]
 
     def __init__(self, directive):
-        # type: (Any) -> None
+        # type: (ObjectDescription) -> None
         self.directive = directive
-        if '_doc_field_type_map' not in directive.__class__.__dict__:
-            directive.__class__._doc_field_type_map = \
-                self.preprocess_fieldtypes(directive.__class__.doc_field_types)
-        self.typemap = directive._doc_field_type_map
+        self.typemap = directive.get_field_type_map()
 
     def preprocess_fieldtypes(self, types):
         # type: (List[Field]) -> Dict[str, Tuple[Field, bool]]
+        warnings.warn('DocFieldTransformer.preprocess_fieldtypes() is deprecated.',
+                      RemovedInSphinx40Warning)
         typemap = {}
         for fieldtype in types:
             for name in fieldtype.names:
