@@ -1,15 +1,13 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.i18n
     ~~~~~~~~~~~~~~~~
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import gettext
-import io
 import os
 import re
 import warnings
@@ -21,19 +19,19 @@ import babel.dates
 from babel.messages.mofile import write_mo
 from babel.messages.pofile import read_po
 
-from sphinx.deprecation import RemovedInSphinx30Warning
+from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
 from sphinx.errors import SphinxError
 from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.matching import Matcher
-from sphinx.util.osutil import SEP, relpath, walk
+from sphinx.util.osutil import SEP, canon_path, relpath
 
 
 logger = logging.getLogger(__name__)
 
 if False:
     # For type annotation
-    from typing import Callable, List, Set  # NOQA
+    from typing import Callable, Generator, List, Set, Tuple  # NOQA
     from sphinx.environment import BuildEnvironment  # NOQA
 
 LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
@@ -43,22 +41,22 @@ class CatalogInfo(LocaleFileInfoBase):
 
     @property
     def po_file(self):
-        # type: () -> unicode
+        # type: () -> str
         return self.domain + '.po'
 
     @property
     def mo_file(self):
-        # type: () -> unicode
+        # type: () -> str
         return self.domain + '.mo'
 
     @property
     def po_path(self):
-        # type: () -> unicode
+        # type: () -> str
         return path.join(self.base_dir, self.po_file)
 
     @property
     def mo_path(self):
-        # type: () -> unicode
+        # type: () -> str
         return path.join(self.base_dir, self.mo_file)
 
     def is_outdated(self):
@@ -68,23 +66,70 @@ class CatalogInfo(LocaleFileInfoBase):
             path.getmtime(self.mo_path) < path.getmtime(self.po_path))
 
     def write_mo(self, locale):
-        # type: (unicode) -> None
-        with io.open(self.po_path, 'rt', encoding=self.charset) as file_po:
+        # type: (str) -> None
+        with open(self.po_path, encoding=self.charset) as file_po:
             try:
                 po = read_po(file_po, locale)
             except Exception as exc:
                 logger.warning(__('reading error: %s, %s'), self.po_path, exc)
                 return
 
-        with io.open(self.mo_path, 'wb') as file_mo:
+        with open(self.mo_path, 'wb') as file_mo:
             try:
                 write_mo(file_mo, po)
             except Exception as exc:
                 logger.warning(__('writing error: %s, %s'), self.mo_path, exc)
 
 
+class CatalogRepository:
+    """A repository for message catalogs."""
+
+    def __init__(self, basedir, locale_dirs, language, encoding):
+        # type: (str, List[str], str, str) -> None
+        self.basedir = basedir
+        self._locale_dirs = locale_dirs
+        self.language = language
+        self.encoding = encoding
+
+    @property
+    def locale_dirs(self):
+        # type: () -> Generator[str, None, None]
+        if not self.language:
+            return
+
+        for locale_dir in self._locale_dirs:
+            locale_dir = path.join(self.basedir, locale_dir)
+            if path.exists(path.join(locale_dir, self.language, 'LC_MESSAGES')):
+                yield locale_dir
+
+    @property
+    def pofiles(self):
+        # type: () -> Generator[Tuple[str, str], None, None]
+        for locale_dir in self.locale_dirs:
+            basedir = path.join(locale_dir, self.language, 'LC_MESSAGES')
+            for root, dirnames, filenames in os.walk(basedir):
+                # skip dot-directories
+                for dirname in dirnames:
+                    if dirname.startswith('.'):
+                        dirnames.remove(dirname)
+
+                for filename in filenames:
+                    if filename.endswith('.po'):
+                        fullpath = path.join(root, filename)
+                        yield basedir, relpath(fullpath, basedir)
+
+    @property
+    def catalogs(self):
+        # type: () -> Generator[CatalogInfo, None, None]
+        for basedir, filename in self.pofiles:
+            domain = canon_path(path.splitext(filename)[0])
+            yield CatalogInfo(basedir, domain, self.encoding)
+
+
 def find_catalog(docname, compaction):
-    # type: (unicode, bool) -> unicode
+    # type: (str, bool) -> str
+    warnings.warn('find_catalog() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if compaction:
         ret = docname.split(SEP, 1)[0]
     else:
@@ -93,22 +138,33 @@ def find_catalog(docname, compaction):
     return ret
 
 
+def docname_to_domain(docname, compation):
+    # type: (str, bool) -> str
+    """Convert docname to domain for catalogs."""
+    if compation:
+        return docname.split(SEP, 1)[0]
+    else:
+        return docname
+
+
 def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
-    # type: (unicode, unicode, List[unicode], unicode, bool) -> List[unicode]
+    # type: (str, str, List[str], str, bool) -> List[str]
+    warnings.warn('find_catalog_files() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if not(lang and locale_dirs):
         return []
 
     domain = find_catalog(docname, compaction)
-    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])  # type: ignore
+    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])
              for dir_ in locale_dirs]
-    files = [relpath(f, srcdir) for f in files if f]  # type: ignore
-    return files  # type: ignore
+    files = [relpath(f, srcdir) for f in files if f]
+    return files
 
 
 def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact=None,
                               charset='utf-8', force_all=False,
                               excluded=Matcher([])):
-    # type: (List[unicode], unicode, List[unicode], bool, unicode, bool, Matcher) -> Set[CatalogInfo]  # NOQA
+    # type: (List[str], str, List[str], bool, str, bool, Matcher) -> Set[CatalogInfo]
     """
     :param list locale_dirs:
        list of path as `['locale_dir1', 'locale_dir2', ...]` to find
@@ -122,9 +178,11 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
        default is False.
     :return: [CatalogInfo(), ...]
     """
+    warnings.warn('find_catalog_source_files() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if gettext_compact is not None:
         warnings.warn('gettext_compact argument for find_catalog_source_files() '
-                      'is deprecated.', RemovedInSphinx30Warning)
+                      'is deprecated.', RemovedInSphinx30Warning, stacklevel=2)
 
     catalogs = set()  # type: Set[CatalogInfo]
 
@@ -140,7 +198,7 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
         if not path.exists(base_dir):
             continue  # locale path is not found
 
-        for dirpath, dirnames, filenames in walk(base_dir, followlinks=True):
+        for dirpath, dirnames, filenames in os.walk(base_dir, followlinks=True):
             filenames = [f for f in filenames if f.endswith('.po')]
             for filename in filenames:
                 if excluded(path.join(relpath(dirpath, base_dir), filename)):
@@ -190,7 +248,7 @@ date_format_mappings = {
     '%x':  'medium',  # Locale’s appropriate date representation.
     '%X':  'medium',  # Locale’s appropriate time representation.
     '%y':  'YY',      # Year without century as a zero-padded decimal number.
-    '%Y':  'YYYY',    # Year with century as a decimal number.
+    '%Y':  'yyyy',    # Year with century as a decimal number.
     '%Z':  'zzzz',    # Time zone name (no characters if no time zone exists).
     '%%':  '%',
 }
@@ -199,7 +257,7 @@ date_format_re = re.compile('(%s)' % '|'.join(date_format_mappings))
 
 
 def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
-    # type: (datetime, unicode, unicode, Callable) -> unicode
+    # type: (datetime, str, str, Callable) -> str
     if locale is None:
         locale = 'en'
 
@@ -220,7 +278,7 @@ def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
 
 
 def format_date(format, date=None, language=None):
-    # type: (str, datetime, unicode) -> unicode
+    # type: (str, datetime, str) -> str
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
         # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
@@ -255,7 +313,7 @@ def format_date(format, date=None, language=None):
 
 
 def get_image_filename_for_language(filename, env):
-    # type: (unicode, BuildEnvironment) -> unicode
+    # type: (str, BuildEnvironment) -> str
     if not env.config.language:
         return filename
 
@@ -275,7 +333,7 @@ def get_image_filename_for_language(filename, env):
 
 
 def search_image_for_language(filename, env):
-    # type: (unicode, BuildEnvironment) -> unicode
+    # type: (str, BuildEnvironment) -> str
     if not env.config.language:
         return filename
 
